package com.alihealth.d2d.provtest;

import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.alihealth.d2d.provtest.utils.Base64;
import com.alihealth.d2d.provtest.utils.EventPojo;
import com.alihealth.d2d.provtest.utils.KeyStoreUtil;
import com.alihealth.d2d.provtest.utils.StreamUtil;
import org.springframework.util.ResourceUtils;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.security.*;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.X509EncodedKeySpec;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * @ClassName ProvTestClient
 * @Desc 省局请求 客户端  请求端
 * @Author houduo.wk
 * @Date 2020/3/13
 **/
public class ProvTestClient {

    /**
     * 请求url
     */
    private static final String url = "http://localhost:1688/prov/test/receive";
    /**
     * 证书地址
     */
    private static final String PFX_FILE_PATH = "D:\\ynyp\\药品追溯\\d2d\\yunnan.pfx";
    /**
     * 证书密码
     */
    private static final String PFX_PASSWORD = "12345678";
    /**
     * 加密算法
     */
    private static final String algorithm = "RSA";


    private static final String XML_DIRECTORY = "D:\\ynyp\\药品追溯\\省市对接（药品v0.19）-通用版\\省市对接（药品v0.19）-通用版\\数据报文示例\\";

    public static void main(String[] args) throws Exception {
        //数据文件内容主业务类型 10：基础信息数据 20：应用信息数据
        //subType数据文件内容子业务类型  详见：数据类型字典表 基本信息(1011-1019、9001-9002)  应用信息（2011-2020、2051-2053）
        testProv("1015");

    }

    public static void testProv(String subType) throws Exception{
        //入参
        String fileName = UUID.randomUUID().toString().replaceAll("-", "").substring(0, 20);
        //需要同步的xml(10-基本信息集 20-应用信息集)
        String data = StreamUtil.getStreamContent(fileName, getData(subType));
        //流水号
        String processId = UUID.randomUUID().toString().replaceAll("-", "").substring(0, 20);
        //事件编号
        String eventId = UUID.randomUUID().toString().replaceAll("-", "").substring(0, 20);
        //数据文件内容主业务类型 10：基础信息数据 20：应用信息数据
        //subType数据文件内容子业务类型  详见：数据类型字典表

        //获取公钥
        String publicKey = getPublicKey();
        System.out.println("publicKey:"+ publicKey);
        String privateKey = getPrivateKey();
        System.out.println("privateKey:"+ privateKey);
        //公钥加密
        String en = testEncrypt(publicKey, data);
        System.out.println("publicKey加密后en:" + en);

        EventPojo eventPojo = new EventPojo();
        eventPojo.setData(en);
        eventPojo.setSubType(subType);

        //请求时间戳
        String timestamp = formatDateToyyyyMMddHHmm(new Date());

        //请求时间戳
        String ver = "1.0";

        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("process_id", processId);
        paramMap.put("event_id", eventId);
        String eventType="10";
        if(subType.startsWith("20")){//20开头
            eventType="20";
        }
        paramMap.put("event_type", eventType);
        paramMap.put("event", JSONObject.toJSONString(eventPojo));
        paramMap.put("timestamp", timestamp);
        paramMap.put("ver", ver);
        Map<String,Object> requestMap= new HashMap<>(4);
        requestMap.put("req", paramMap);
        System.out.println(JSONObject.toJSONString(requestMap));
        String body = HttpRequest.post(url).body(JSONObject.toJSONString(requestMap)).timeout(10000).execute().body();
        System.out.println(body);


    }

    /**
     * 公钥加密
     * @param key
     * @param data
     * @return
     * @throws NoSuchAlgorithmException
     * @throws InvalidKeySpecException
     * @throws NoSuchPaddingException
     * @throws IllegalBlockSizeException
     * @throws BadPaddingException
     * @throws InvalidKeyException
     * @throws IOException
     */
    public static String testEncrypt(String key,String data) throws NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException, IllegalBlockSizeException, BadPaddingException, InvalidKeyException, IOException{
        byte[] decode = java.util.Base64.getDecoder().decode(key);
        RSAPublicKey pubKey = (RSAPublicKey) KeyFactory.getInstance(algorithm).generatePublic(new X509EncodedKeySpec(decode));
        //RSA加密
        Cipher ci = Cipher.getInstance(algorithm);
        ci.init(Cipher.ENCRYPT_MODE, pubKey);


        byte[] bytes = data.getBytes();
        int inputLen = bytes.length;
        int offLen = 0;//偏移量
        int i = 0;
        ByteArrayOutputStream bops = new ByteArrayOutputStream();
        while(inputLen - offLen > 0){
            byte [] cache;
            if(inputLen - offLen > 117){
                cache = ci.doFinal(bytes, offLen,117);
            }else{
                cache = ci.doFinal(bytes, offLen,inputLen - offLen);
            }
            bops.write(cache);
            i++;
            offLen = 117 * i;
        }
        bops.close();
        byte[] encryptedData = bops.toByteArray();
        String encodeToString = java.util.Base64.getEncoder().encodeToString(encryptedData);
        return encodeToString;
    }

    /**
     * 获取私钥
     * @return
     * @throws Exception
     */
    private static String getPrivateKey() throws Exception {
        KeyStore keyStore = KeyStoreUtil.loadKetStore(PFX_FILE_PATH,PFX_PASSWORD);
        return Base64.encode(KeyStoreUtil.getPrivateKey(keyStore, PFX_PASSWORD).getEncoded());
    }

    /**
     * 获取公钥
     * @return
     * @throws Exception
     */
    private static String getPublicKey() throws Exception {
        KeyStore keyStore = KeyStoreUtil.loadKetStore(PFX_FILE_PATH,PFX_PASSWORD);
        PublicKey publicKey = KeyStoreUtil.getPublicKey(keyStore);
        return Base64.encode(publicKey.getEncoded());
    }
    /**
     * 返回时间字符串, 可读形式的, yy年M月d日HH:mm 格式.
     *
     * @return - String 格式化后的时间
     */
    public static String formatDateToyyyyMMddHHmm(Date date) {
        if (date == null) {
            return "";
        }

        SimpleDateFormat dateFormat = new SimpleDateFormat(
                "yyyyMMddHHmmss");

        return dateFormat.format(date);
    }

    /**
     * 动态获取文件内容
     * @param subType
     * @return
     * @throws IOException
     */
    private static String getData(String subType) throws IOException {
        // 构建文件名
        String fileName = subType + ".xml";
        // 获取文件路径
        File file = ResourceUtils.getFile(XML_DIRECTORY + fileName);
        Path path = Paths.get(file.getAbsolutePath());

        // 读取文件内容并返回
        return new String(Files.readAllBytes(path));

    }



}
